% Matlab file for_ paper "Solving Heterogeneous Agent Models by Projection and Perturbation"
% Michael Reiter, Institute for Advanced Studies, September 2006
% Last update: June 2008
% Feel free to use, copy and modify at your own risk;
%   this program comes with NO WARRANTY WHATSOEVER
% Output:
%   betaVAR:   coefficient of the VAR model in moments; 
%              matrices [A B] in Equ.(55); last two columns refer to shocks
%              4x1 cell array, for_ VAR of dimension 1-4
%   err_VAR:   covariance matrix of the error of VAR model
%   err_spl:   covariance matrix of the error of the reduced model with splines
function [err_VAR,err_spl,betaVAR] = dosimul
  % Simulate the model:
  global Params;


  % set standard deviations of the two shocks:
  sigZ = 0.014;
  sigT = 0.020;
  Sigma = diag([sigZ sigT].^2);
  
  % load parameters for_ "Exact Solution":
  load('ststres','par','Kequ','Params2');
  Params = Params2;
  Params.ndstst = 1000;

  Params.SmoothDev = 0;
  hetsolve;  %just to set some parameters!
  load('simsres_ex','par','Kequ','D','G1','impact','abseig');
  Params.Dstst = D;  %steady state distribution
  nMom = 4;  %compute results for_ up to 4 moments;

  [betaVAR,SExact,HExact,err_VAR] = dovar(G1,impact,Sigma);

  G1Exact = G1;
  impactExact = impact;
  Params.SmoothDev = 1;
  h = HExact;

  % number of spline bases to be used for_ reduced model:
  Params.nSpline = 20;
  hetsolve;
  disp(sprintf('nSpline = %d',Params.nSpline));
  load('simsres_spl','par','Kequ','D','G1','impact','abseig');
  % compute unconditional error for_ reduced model:
  n = size(G1Exact,1);
  na = size(G1,1);
  GG = [G1Exact zeros(n,na);
	zeros(na,n) G1];
  II = [impactExact;impact];
  h2 = make_h(1:4,length(G1));
  HH = [h;-h2];
  SigmaBig = lyapsym(GG,II,Sigma);
  err_spl = HH'*SigmaBig*HH;
