% Matlab file for_ paper "Solving Heterogeneous Agent Models by Projection and Perturbation"
% Michael Reiter, Institute for Advanced Studies, September 2006
% Last update: June 2008
% Feel free to use, copy and modify at your own risk;
%   this program comes with NO WARRANTY WHATSOEVER
%Inputs:
%  G1:       as output from Sims', for_ exact model
%  impact:   same
%  Sigma:    variance-covariance matrix of shocks
%Outputs:
%  betaex:   coefficients of VAR
%  SigmaX:   CovMatrix of exact model
%  Hen:      matrix that generates first 4 moments in exact model
%  CovError: 4x1 cell array, covariance matrix of unconditional error for_ VAR with
%               1-4 moments
function [betaex,SigmaX,Hen,CovError] = domomexact(G1,impact,Sigma)
  n = size(G1,1);

  % matrix to extract exogenous variables:
  Hex = [zeros(n-2,2);eye(2)];
  % matrix to generate first 4 moments:
  Hen = make_h(1:4,n);

  % CovMatrix of exact model:
  SigmaX = lyapsym(G1,impact,Sigma);
  for j=1:4  %up to 4 moments:
    h = Hen(:,1:j);
    H = [h Hex];
    SH = SigmaX*H;
    aux1 = (H'*G1*SH)/(H'*SH);
    aux2 = H'*impact;
    betaex{j} = [aux1 aux2];
    if(nargout>3)
      h2 = [eye(j);zeros(2,j)];

      n = size(G1,1);
      GG = [G1 zeros(n,j+2);
	    zeros(j+2,n) aux1];
      II = [impact;aux2];
      HH = [h;-h2];
      SigmaBig = lyapsym(GG,II,Sigma); % could be done more efficiently!
      CovError{j} = HH'*SigmaBig*HH;
    end
  end
  