% Matlab file for_ paper "Solving Heterogeneous Agent Models by Projection and Perturbation"
% Michael Reiter, Institute for Advanced Studies, September 2006
% Last update: June 2008
% Feel free to use, copy and modify at your own risk;
%   this program comes with NO WARRANTY WHATSOEVER
function resid = equ(X)
  global Params;

  xcurr = X(Params.ix);
  xlag = X(Params.ixlag);
  eta = X(Params.ieta);
  eps = X(Params.ieps);

  [D,par,dz,dtau] = x2par(xcurr);
  [Dlag,parlag,dzlag,dtaulag] = x2par(xlag);

  [R,wage,Transf] = factorprices(Dlag,dz,dtau);

  if(Params.iKS)  %with Krusell/Smith method, dynamics of moments exogenously given:
    resD = -(D-Params.MomStst) + Params.Aks*([Dlag-Params.MomStst;dzlag;dtaulag]) + Params.Bks*[eps];
  else
    Pi = forwardmat(par,Transf,R,wage,Params.knotDistrK);
    D2 = forward(Dlag,Pi);
    resD = distr2par(D2) - distr2par(D);
  end
  resC = eulerres(parlag,par,R,wage,Transf) + eta;
  resid = [resD;
	   resC;
	   dz-Params.rhoz*dzlag-eps(1);
	   dtau-Params.rhot*dtaulag-eps(2)];
    
