% Matlab file for_ paper "Solving Heterogeneous Agent Models by Projection and Perturbation"
% Michael Reiter, Institute for Advanced Studies, September 2006
% Last update: June 2008
% Feel free to use, copy and modify at your own risk;
%   this program comes with NO WARRANTY WHATSOEVER
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% FUNCTION THAT DEFINES EULER RESIDUALS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% inputs:
%   par:           parameter vector for_ current savings function_
%   parnext:       parameter vector for_ next periods' savings function_
%   R,wage,Transf: 1+interest,wage,transfers
%   xstart:        grid at which_ to compute Euler residuals; default: knot points of spline
% output:
function z = eulerres(par,parnext,R,wage,Transf,xstart)
  global Params;

  [x,s,s2,xcrit] = savingspline(par);
  if(xcrit<=0 | any(s>x))  %signal inadmissible value to routine 'broydn';
    z = 1e100;
    return;
  end

  if(nargin>5) % xstart given
    sstart = interp_cubspline(x,s,s2,xstart);
    C = xstart-sstart;
    assets = R*sstart + Transf;
    xnext = repmat(assets,1,size(Params.xY,1)) + wage*repmat(Params.xY',size(xstart,1),1);
  else % take x as starting value for_ cash on hand
    C = x-s;
    assets = R*s + Transf;
    xnext = repmat(assets,1,size(Params.xY,1)) + wage*repmat(Params.xY',size(x,1),1);
  end

  [xn,sn,s2n,xcritn] = savingspline(parnext);
  if(xcritn<=0 | any(sn>xn))
    z = 1e100;
    return;
  end
  ss = interp_cubspline(xn,sn,s2n,xnext);  %saving tomorrow
  Cnext = xnext - ss;  %consumption tomorrow

  MUnext = margutil(Cnext);
  MUexp = MUnext*Params.wY;  %sum up with quadrature weights
  MU = margutil(C);

  % Euler residual:
  % expressed in relative consumption units:
  z = 1 - invmargutil(Params.beta*R*MUexp)./C;
