% Matlab file for_ paper "Solving Heterogeneous Agent Models by Projection and Perturbation"
% Michael Reiter, Institute for Advanced Studies, September 2006
% Last update: June 2008
% Feel free to use, copy and modify at your own risk;
%   this program comes with NO WARRANTY WHATSOEVER
% 
% Compute j-th central moment of the cross-sectional distribution of k,
% Inputs:
%   pvec:  vector of probabilities
%          if empty: function returns integrator rather than integral
%   j:     which_ moment
%   k:     vector of k-nodes; default: Params.knotDistrK;
function EV = expect_k(pvec,j,k)
  global Params;
  if(nargin==1)
    j = 1;
  end
  if(nargin<3)
    k = Params.knotDistrK;
  end
  doCentral = 0;
  if(j<-1)
    doCentral = 1;
  end
  j = abs(j);


  subtr = 0;
  if(doCentral) %compute central moment
    subtr = expect_k(pvec,1,k);
  end
  
  % subtract mean if j>1:
  k = k-subtr;
  y = (k.^(j+1))./(j+1);
  dy = [k(1)^j;diff(y)./diff(k)];
  if(isempty(pvec))  %return integrating vector:
    EV = dy;
  else
    EV = dy'*pvec;
  end
  
