% Matlab file for_ paper "Solving Heterogeneous Agent Models by Projection and Perturbation"
% Michael Reiter, Institute for Advanced Studies, September 2006
% Last update: June 2008
% Feel free to use, copy and modify at your own risk;
%   this program comes with NO WARRANTY WHATSOEVER
function Pi = forwardmat(par,Transf,R,wage,knotDistrK);
  [x,s,s2,xcrit] = savingspline(par);
  XofK = rootspline(x,s,s2,knotDistrK);
  Pi = transprobmat(Transf,R,wage,XofK,knotDistrK);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% FUNCTION TO COMPUTE TRANSITION PROBABILITY MATRIX
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function Pi = transprobmat(Transf,R,w,XofK,knotDistrK)
  global Params;
  nd = length(knotDistrK);
  Pi = zeros(nd,nd);

  xx = XofK - Transf - R*Params.kmin;
  dd = disty(xx,w);
  Pi(:,1) = [dd(1);diff(dd)];
  idk = (w/R)./diff(knotDistrK);
  for i=1:nd
    xx = XofK(i) - Transf - R*knotDistrK;
    dd = idisty(xx,w);
    intF = -(diff(dd).*idk)';
    if(i==1)
      Pi(1,2:nd) = intF;
    else
      if(any(intF<intFlast-1e-8))
	error('transition matrix not positive');
      end
      intF = max(intF,intFlast);
      Pi(i,2:nd) = intF-intFlast;
    end
    intFlast = intF;
  end 

  % DO THE NEXT OEPRATIONS COLUMNWISE, TO SAVE MEMORY:
  nBIG = round(nd/100);
  for i=1:nd
    m = Pi(:,i);
    if(i>=nd-nBIG)
      %necessary for_ last intervals, to capture mass beyond last knot point:
      m(nd) = 1-sum(m);
    end
    m = m/sum(m);
    Pi(:,i) = m;
  end
