% Matlab file for_ paper "Solving Heterogeneous Agent Models by Projection and Perturbation"
% Michael Reiter, IHS, September 2006
% Last update: June 2008
% Feel free to use, copy and modify at your own risk;
%   this program comes with NO WARRANTY WHATSOEVER
% Input: 
%   par: finite parameterization of the consumption function
% Outputs:  
%   x:   knot points of spline
%   s:   saving at knot points
%   s2:  second derivative of saving at knot points
% The ouputs define a spline approximation for the consumption (or savings) function
function [x,s,s2,xcrit] = savingsspline(par)
  global Params;
  xcrit = par(1);
  x = [xcrit ; xcrit+Params.knotXi];
  s = [0;par(2:end)];  
  % s2 = natcubspline(x,s);

  % compute approximation do first derivative at x(1):
  dy = diff(s(1:3))./diff(x(1:3));
  ddy = diff(dy) ./ diff(midpoint(x(1:3)));
  dy1 = dy(1) - ddy*(x(2)-x(1))/2;
  % impose that spline approximation has this derivative at x(1):
  s2 = natcubspline(x,s,dy1);

function m = midpoint(x)
  n = length(x);
  m = 0.5*(x(1:n-1)+x(2:n));

